@@ -1,6 +1,6 @@
 /* Advanced array manipulation routines for S-Lang */
 /*
-Copyright (C) 2004-2010 John E. Davis
+Copyright (C) 2004-2017,2018 John E. Davis
 
 This file is part of the S-Lang Library.
 
@@ -36,10 +36,10 @@ static int next_transposed_index (SLindex_Type *dims, SLindex_Type *max_dims, un
 
    for (i = 0; i < (int) num_dims; i++)
      {
-	int dims_i;
+	SLindex_Type dims_i;
 
 	dims_i = dims [i] + 1;
-	if (dims_i != (int) max_dims [i])
+	if (dims_i != max_dims [i])
 	  {
 	     dims [i] = dims_i;
 	     return 0;
@@ -102,6 +102,10 @@ static int check_for_empty_array (SLCONST char *fun, unsigned int num)
 #define ALL_FUNCTION all_floats
 #define IS_NAN_FUNCTION _pSLmath_isnan
 #define ABS_FUNCTION fabs
+#define WHEREFIRSTMIN_FUNC wherefirstmin_float
+#define WHEREFIRSTMAX_FUNC wherefirstmax_float
+#define WHERELASTMIN_FUNC wherelastmin_float
+#define WHERELASTMAX_FUNC wherelastmax_float
 #include "slarrfun.inc"
 
 /* -------------- DOUBLE --------------------- */
@@ -130,6 +134,10 @@ static int check_for_empty_array (SLCONST char *fun, unsigned int num)
 #define ALL_FUNCTION all_doubles
 #define IS_NAN_FUNCTION _pSLmath_isnan
 #define ABS_FUNCTION fabs
+#define WHEREFIRSTMIN_FUNC wherefirstmin_double
+#define WHEREFIRSTMAX_FUNC wherefirstmax_double
+#define WHERELASTMIN_FUNC wherelastmin_double
+#define WHERELASTMAX_FUNC wherelastmax_double
 #include "slarrfun.inc"
 
 #define GENERIC_TYPE_A double
@@ -169,6 +177,10 @@ static int check_for_empty_array (SLCONST char *fun, unsigned int num)
 #define ANY_FUNCTION any_ints
 #define ALL_FUNCTION all_ints
 #define ABS_FUNCTION abs
+#define WHEREFIRSTMIN_FUNC wherefirstmin_int
+#define WHEREFIRSTMAX_FUNC wherefirstmax_int
+#define WHERELASTMIN_FUNC wherelastmin_int
+#define WHERELASTMAX_FUNC wherelastmax_int
 #include "slarrfun.inc"
 
 /* -------------- UNSIGNED INT --------------------- */
@@ -180,6 +192,10 @@ static int check_for_empty_array (SLCONST char *fun, unsigned int num)
 #define MAX_FUNCTION max_uints
 #define ANY_FUNCTION any_uints
 #define ALL_FUNCTION all_uints
+#define WHEREFIRSTMIN_FUNC wherefirstmin_uint
+#define WHEREFIRSTMAX_FUNC wherefirstmax_uint
+#define WHERELASTMIN_FUNC wherelastmin_uint
+#define WHERELASTMAX_FUNC wherelastmax_uint
 #include "slarrfun.inc"
 
 #if SIZEOF_LONG != SIZEOF_INT
@@ -198,6 +214,10 @@ static int check_for_empty_array (SLCONST char *fun, unsigned int num)
 # define ANY_FUNCTION any_longs
 # define ALL_FUNCTION all_longs
 # define ABS_FUNCTION labs
+# define WHEREFIRSTMIN_FUNC wherefirstmin_long
+# define WHEREFIRSTMAX_FUNC wherefirstmax_long
+# define WHERELASTMIN_FUNC wherelastmin_long
+# define WHERELASTMAX_FUNC wherelastmax_long
 # include "slarrfun.inc"
 /* -------------- UNSIGNED LONG --------------------- */
 # define GENERIC_TYPE unsigned long
@@ -208,6 +228,10 @@ static int check_for_empty_array (SLCONST char *fun, unsigned int num)
 # define MAX_FUNCTION max_ulongs
 # define ANY_FUNCTION any_ulongs
 # define ALL_FUNCTION all_ulongs
+# define WHEREFIRSTMIN_FUNC wherefirstmin_ulong
+# define WHEREFIRSTMAX_FUNC wherefirstmax_ulong
+# define WHERELASTMIN_FUNC wherelastmin_ulong
+# define WHERELASTMAX_FUNC wherelastmax_ulong
 # include "slarrfun.inc"
 #else
 # define transpose_longs transpose_ints
@@ -226,6 +250,14 @@ static int check_for_empty_array (SLCONST char *fun, unsigned int num)
 # define any_ulongs any_uints
 # define all_longs all_ints
 # define all_ulongs all_uints
+# define wherefirstmax_long wherefirstmax_int
+# define wherefirstmin_long wherefirstmin_int
+# define wherelastmin_long wherelastmin_int
+# define wherelastmax_long wherelastmax_int
+# define wherefirstmax_ulong wherefirstmax_uint
+# define wherefirstmin_ulong wherefirstmin_uint
+# define wherelastmin_ulong wherelastmin_uint
+# define wherelastmax_ulong wherelastmax_uint
 #endif
 
 #if SIZEOF_SHORT != SIZEOF_INT
@@ -242,6 +274,10 @@ static int check_for_empty_array (SLCONST char *fun, unsigned int num)
 # define ANY_FUNCTION any_shorts
 # define ALL_FUNCTION all_shorts
 # define ABS_FUNCTION abs
+# define WHEREFIRSTMIN_FUNC wherefirstmin_short
+# define WHEREFIRSTMAX_FUNC wherefirstmax_short
+# define WHERELASTMIN_FUNC wherelastmin_short
+# define WHERELASTMAX_FUNC wherelastmax_short
 # include "slarrfun.inc"
 /* -------------- UNSIGNED SHORT --------------------- */
 # define GENERIC_TYPE unsigned short
@@ -252,6 +288,10 @@ static int check_for_empty_array (SLCONST char *fun, unsigned int num)
 # define MAX_FUNCTION max_ushorts
 # define ANY_FUNCTION any_ushorts
 # define ALL_FUNCTION all_ushorts
+# define WHEREFIRSTMIN_FUNC wherefirstmin_ushort
+# define WHEREFIRSTMAX_FUNC wherefirstmax_ushort
+# define WHERELASTMIN_FUNC wherelastmin_ushort
+# define WHERELASTMAX_FUNC wherelastmax_ushort
 # include "slarrfun.inc"
 #else
 # define transpose_shorts transpose_ints
@@ -269,10 +309,18 @@ static int check_for_empty_array (SLCONST char *fun, unsigned int num)
 # define any_ushorts any_uints
 # define all_shorts all_ints
 # define all_ushorts all_uints
+# define wherefirstmax_short wherefirstmax_int
+# define wherefirstmin_short wherefirstmin_int
+# define wherelastmin_short wherelastmin_int
+# define wherelastmax_short wherelastmax_int
+# define wherefirstmax_ushort wherefirstmax_uint
+# define wherefirstmin_ushort wherefirstmin_uint
+# define wherelastmin_ushort wherelastmin_uint
+# define wherelastmax_ushort wherelastmax_uint
 #endif
 
 /* -------------- CHAR --------------------- */
-#define GENERIC_TYPE char
+#define GENERIC_TYPE signed char
 #define TRANSPOSE_2D_ARRAY transpose_chars
 #define SUM_FUNCTION sum_chars
 #define SUMSQ_FUNCTION sumsq_chars
@@ -284,6 +332,10 @@ static int check_for_empty_array (SLCONST char *fun, unsigned int num)
 #define ANY_FUNCTION any_chars
 #define ALL_FUNCTION all_chars
 #define ABS_FUNCTION abs
+#define WHEREFIRSTMIN_FUNC wherefirstmin_char
+#define WHEREFIRSTMAX_FUNC wherefirstmax_char
+#define WHERELASTMIN_FUNC wherelastmin_char
+#define WHERELASTMAX_FUNC wherelastmax_char
 #include "slarrfun.inc"
 /* -------------- UNSIGNED CHAR --------------------- */
 #define GENERIC_TYPE unsigned char
@@ -294,6 +346,10 @@ static int check_for_empty_array (SLCONST char *fun, unsigned int num)
 #define MAX_FUNCTION max_uchars
 #define ANY_FUNCTION any_uchars
 #define ALL_FUNCTION all_uchars
+#define WHEREFIRSTMIN_FUNC wherefirstmin_uchar
+#define WHEREFIRSTMAX_FUNC wherefirstmax_uchar
+#define WHERELASTMIN_FUNC wherelastmin_uchar
+#define WHERELASTMAX_FUNC wherelastmax_uchar
 #include "slarrfun.inc"
 
 #ifdef HAVE_LONG_LONG
@@ -308,6 +364,10 @@ static int check_for_empty_array (SLCONST char *fun, unsigned int num)
 #  define ANY_FUNCTION any_llongs
 #  define ALL_FUNCTION all_llongs
 #  define ABS_FUNCTION(x) (((x)>=0)?(x):-(x))
+#  define WHEREFIRSTMIN_FUNC wherefirstmin_llong
+#  define WHEREFIRSTMAX_FUNC wherefirstmax_llong
+#  define WHERELASTMIN_FUNC wherelastmin_llong
+#  define WHERELASTMAX_FUNC wherelastmax_llong
 #  include "slarrfun.inc"
 /* -------------- UNSIGNED LONG --------------------- */
 #  define GENERIC_TYPE unsigned long long
@@ -315,11 +375,15 @@ static int check_for_empty_array (SLCONST char *fun, unsigned int num)
 #  define MAX_FUNCTION max_ullongs
 #  define ANY_FUNCTION any_ullongs
 #  define ALL_FUNCTION all_ullongs
+#  define WHEREFIRSTMIN_FUNC wherefirstmin_ullong
+#  define WHEREFIRSTMAX_FUNC wherefirstmax_ullong
+#  define WHERELASTMIN_FUNC wherelastmin_ullong
+#  define WHERELASTMAX_FUNC wherelastmax_ullong
 #  include "slarrfun.inc"
 # else
 #  define transpose_llongs transpose_longs
 #  define min_llongs min_longs
-#  define minabs_llongs minabs_llongs
+#  define minabs_llongs minabs_longs
 #  define min_ullongs min_ullongs
 #  define max_llongs max_llongs
 #  define maxabs_llongs maxabs_llongs
@@ -328,9 +392,84 @@ static int check_for_empty_array (SLCONST char *fun, unsigned int num)
 #  define any_ullongs any_ullongs
 #  define all_llongs all_llongs
 #  define all_ullongs all_ullongs
+#  define wherefirstmax_llong wherefirstmax_long
+#  define wherefirstmin_llong wherefirstmin_long
+#  define wherelastmin_llong wherelastmin_long
+#  define wherelastmax_llong wherelastmax_long
+#  define wherefirstmax_ullong wherefirstmax_ulong
+#  define wherefirstmin_ullong wherefirstmin_ulong
+#  define wherelastmin_ullong wherelastmin_ulong
+#  define wherelastmax_ullong wherelastmax_ulong
 # endif
 #endif				       /* HAVE_LONG_LONG */
 
+#define GENERIC_TYPE_A signed char
+#define GENERIC_TYPE_B double
+#define DO_WHEREFIRST_OP_FUNC wherefirst_op_char
+#define DO_WHERELAST_OP_FUNC wherelast_op_char
+#include "slarrfun.inc"
+#define GENERIC_TYPE_A unsigned char
+#define GENERIC_TYPE_B double
+#define DO_WHEREFIRST_OP_FUNC wherefirst_op_uchar
+#define DO_WHERELAST_OP_FUNC wherelast_op_uchar
+#include "slarrfun.inc"
+
+#define GENERIC_TYPE_A _pSLint16_Type
+#define GENERIC_TYPE_B double
+#define DO_WHEREFIRST_OP_FUNC wherefirst_op_int16
+#define DO_WHERELAST_OP_FUNC wherelast_op_int16
+#include "slarrfun.inc"
+#define GENERIC_TYPE_A _pSLuint16_Type
+#define GENERIC_TYPE_B double
+#define DO_WHEREFIRST_OP_FUNC wherefirst_op_uint16
+#define DO_WHERELAST_OP_FUNC wherelast_op_uint16
+#include "slarrfun.inc"
+
+#define GENERIC_TYPE_A _pSLint32_Type
+#define GENERIC_TYPE_B double
+#define DO_WHEREFIRST_OP_FUNC wherefirst_op_int32
+#define DO_WHERELAST_OP_FUNC wherelast_op_int32
+#include "slarrfun.inc"
+#define GENERIC_TYPE_A _pSLuint32_Type
+#define GENERIC_TYPE_B double
+#define DO_WHEREFIRST_OP_FUNC wherefirst_op_uint32
+#define DO_WHERELAST_OP_FUNC wherelast_op_uint32
+#include "slarrfun.inc"
+
+#if _pSLANG_INT64_TYPE
+# define GENERIC_TYPE_A _pSLint64_Type
+# define GENERIC_TYPE_B double
+# define DO_WHEREFIRST_OP_FUNC wherefirst_op_int64
+# define DO_WHERELAST_OP_FUNC wherelast_op_int64
+# include "slarrfun.inc"
+# define GENERIC_TYPE_A _pSLuint64_Type
+# define GENERIC_TYPE_B double
+# define DO_WHEREFIRST_OP_FUNC wherefirst_op_uint64
+# define DO_WHERELAST_OP_FUNC wherelast_op_uint64
+# include "slarrfun.inc"
+# define GENERIC_TYPE_A _pSLint64_Type
+# define GENERIC_TYPE_B _pSLint64_Type
+# define DO_WHEREFIRST_OP_FUNC wherefirst_op_int64_alt
+# define DO_WHERELAST_OP_FUNC wherelast_op_int64_alt
+# include "slarrfun.inc"
+# define GENERIC_TYPE_A _pSLuint64_Type
+# define GENERIC_TYPE_B _pSLuint64_Type
+# define DO_WHEREFIRST_OP_FUNC wherefirst_op_uint64_alt
+# define DO_WHERELAST_OP_FUNC wherelast_op_uint64_alt
+# include "slarrfun.inc"
+#endif
+
+#define GENERIC_TYPE_A float
+#define GENERIC_TYPE_B double
+#define DO_WHEREFIRST_OP_FUNC wherefirst_op_float
+#define DO_WHERELAST_OP_FUNC wherelast_op_float
+#include "slarrfun.inc"
+#define GENERIC_TYPE_A double
+#define GENERIC_TYPE_B double
+#define DO_WHEREFIRST_OP_FUNC wherefirst_op_double
+#define DO_WHERELAST_OP_FUNC wherelast_op_double
+#include "slarrfun.inc"
+
 /* This routine works only with linear arrays */
 static SLang_Array_Type *transpose (SLang_Array_Type *at)
 {
@@ -350,6 +489,7 @@ static SLang_Array_Type *transpose (SLang_Array_Type *at)
        || (num_dims == 1))
      {
 	bt = SLang_duplicate_array (at);
+	if (bt == NULL) return NULL;
 	if (num_dims == 1) bt->num_dims = 2;
 	goto transpose_dims;
      }
@@ -429,7 +569,7 @@ static void array_transpose (SLang_Array_Type *at)
 
 #if SLANG_HAS_FLOAT
 static int get_inner_product_parms (SLang_Array_Type *a, int *dp,
-				    unsigned int *loops, unsigned int *other)
+				    SLuindex_Type *loops, SLuindex_Type *other)
 {
    int num_dims;
    int d;
@@ -476,14 +616,14 @@ static void do_inner_product (void)
 {
    SLang_Array_Type *a, *b, *c;
    void (*fun)(SLang_Array_Type *, SLang_Array_Type *, SLang_Array_Type *,
-	       unsigned int, unsigned int, unsigned int, unsigned int,
-	       unsigned int);
+	       SLuindex_Type, SLuindex_Type, SLuindex_Type, SLuindex_Type,
+	       SLuindex_Type);
    SLtype c_type;
    SLindex_Type dims[SLARRAY_MAX_DIMS];
    int status;
-   unsigned int a_loops, b_loops, b_inc, a_stride;
+   SLuindex_Type a_loops, b_loops, b_inc, a_stride;
    int ai_dims, i, j;
-   unsigned int num_dims, a_num_dims, b_num_dims;
+   SLuindex_Type num_dims, a_num_dims, b_num_dims;
    int ai, bi;
 
    /* The result of a inner_product will be either a float, double, or
@@ -938,8 +1078,8 @@ static int sum_complex (VOID_STAR zp, unsigned int inc, unsigned int num, VOID_S
 	z += inc;
      }
    s = (double *)sp;
-   s[0] = sr;
-   s[1] = si;
+   s[0] = sr + sr_err;
+   s[1] = si + si_err;
    return 0;
 }
 
@@ -1014,7 +1154,7 @@ static int prod_complex (VOID_STAR zp, unsigned int inc, unsigned int num, VOID_
 	double a = sr, b = si;
 	double c = z[0], d = z[1];
 	sr = (a*c-b*d);
-	si = (b*c-a*d);
+	si = (b*c+a*d);
 	z += inc;
      }
    s = (double *)sp;
@@ -1150,6 +1290,122 @@ array_max (void)
    (void) SLarray_contract_array (Array_Max_Funs);
 }
 
+/* The wherexxx function do not support the optional dim argument */
+#define DO_WHEREXXX_YYY(what, tbl) \
+   if (SLang_Num_Function_Args != 1) \
+       _pSLang_verror (SL_USAGE_ERROR, "Usage: idx = %s(array)", what); \
+   else \
+     (void) SLarray_contract_array (tbl);
+
+static SLCONST SLarray_Contract_Type Array_WhereFirstMin_Funs [] =
+{
+   {SLANG_CHAR_TYPE, SLANG_CHAR_TYPE, SLANG_ARRAY_INDEX_TYPE, (SLarray_Contract_Fun_Type *) wherefirstmin_char},
+   {SLANG_UCHAR_TYPE, SLANG_UCHAR_TYPE, SLANG_ARRAY_INDEX_TYPE, (SLarray_Contract_Fun_Type *) wherefirstmin_uchar},
+   {SLANG_SHORT_TYPE, SLANG_SHORT_TYPE, SLANG_ARRAY_INDEX_TYPE, (SLarray_Contract_Fun_Type *) wherefirstmin_short},
+   {SLANG_USHORT_TYPE, SLANG_USHORT_TYPE, SLANG_ARRAY_INDEX_TYPE, (SLarray_Contract_Fun_Type *) wherefirstmin_ushort},
+   {SLANG_INT_TYPE, SLANG_INT_TYPE, SLANG_ARRAY_INDEX_TYPE, (SLarray_Contract_Fun_Type *) wherefirstmin_int},
+   {SLANG_UINT_TYPE, SLANG_UINT_TYPE, SLANG_ARRAY_INDEX_TYPE, (SLarray_Contract_Fun_Type *) wherefirstmin_uint},
+   {SLANG_LONG_TYPE, SLANG_LONG_TYPE, SLANG_ARRAY_INDEX_TYPE, (SLarray_Contract_Fun_Type *) wherefirstmin_long},
+   {SLANG_ULONG_TYPE, SLANG_ULONG_TYPE, SLANG_ARRAY_INDEX_TYPE, (SLarray_Contract_Fun_Type *) wherefirstmin_ulong},
+#if defined(HAVE_LONG_LONG) && (SIZEOF_LONG_LONG != SIZEOF_LONG)
+   {SLANG_LLONG_TYPE, SLANG_LLONG_TYPE, SLANG_ARRAY_INDEX_TYPE, (SLarray_Contract_Fun_Type *) wherefirstmin_llong},
+   {SLANG_ULLONG_TYPE, SLANG_ULLONG_TYPE, SLANG_ARRAY_INDEX_TYPE, (SLarray_Contract_Fun_Type *) wherefirstmin_ullong},
+#endif
+#if SLANG_HAS_FLOAT
+   {SLANG_FLOAT_TYPE, SLANG_FLOAT_TYPE, SLANG_ARRAY_INDEX_TYPE, (SLarray_Contract_Fun_Type *) wherefirstmin_float},
+   {SLANG_DOUBLE_TYPE, SLANG_DOUBLE_TYPE, SLANG_ARRAY_INDEX_TYPE, (SLarray_Contract_Fun_Type *) wherefirstmin_double},
+#endif
+     {0, 0, 0, NULL}
+};
+
+static void
+array_wherefirstmin (void)
+{
+   DO_WHEREXXX_YYY("wherefirstmin", Array_WhereFirstMin_Funs);
+}
+
+static SLCONST SLarray_Contract_Type Array_WhereFirstMax_Funs [] =
+{
+   {SLANG_CHAR_TYPE, SLANG_CHAR_TYPE, SLANG_ARRAY_INDEX_TYPE, (SLarray_Contract_Fun_Type *) wherefirstmax_char},
+   {SLANG_UCHAR_TYPE, SLANG_UCHAR_TYPE, SLANG_ARRAY_INDEX_TYPE, (SLarray_Contract_Fun_Type *) wherefirstmax_uchar},
+   {SLANG_SHORT_TYPE, SLANG_SHORT_TYPE, SLANG_ARRAY_INDEX_TYPE, (SLarray_Contract_Fun_Type *) wherefirstmax_short},
+   {SLANG_USHORT_TYPE, SLANG_USHORT_TYPE, SLANG_ARRAY_INDEX_TYPE, (SLarray_Contract_Fun_Type *) wherefirstmax_ushort},
+   {SLANG_INT_TYPE, SLANG_INT_TYPE, SLANG_ARRAY_INDEX_TYPE, (SLarray_Contract_Fun_Type *) wherefirstmax_int},
+   {SLANG_UINT_TYPE, SLANG_UINT_TYPE, SLANG_ARRAY_INDEX_TYPE, (SLarray_Contract_Fun_Type *) wherefirstmax_uint},
+   {SLANG_LONG_TYPE, SLANG_LONG_TYPE, SLANG_ARRAY_INDEX_TYPE, (SLarray_Contract_Fun_Type *) wherefirstmax_long},
+   {SLANG_ULONG_TYPE, SLANG_ULONG_TYPE, SLANG_ARRAY_INDEX_TYPE, (SLarray_Contract_Fun_Type *) wherefirstmax_ulong},
+#if defined(HAVE_LONG_LONG) && (SIZEOF_LONG_LONG != SIZEOF_LONG)
+   {SLANG_LLONG_TYPE, SLANG_LLONG_TYPE, SLANG_ARRAY_INDEX_TYPE, (SLarray_Contract_Fun_Type *) wherefirstmax_llong},
+   {SLANG_ULLONG_TYPE, SLANG_ULLONG_TYPE, SLANG_ARRAY_INDEX_TYPE, (SLarray_Contract_Fun_Type *) wherefirstmax_ullong},
+#endif
+#if SLANG_HAS_FLOAT
+   {SLANG_FLOAT_TYPE, SLANG_FLOAT_TYPE, SLANG_ARRAY_INDEX_TYPE, (SLarray_Contract_Fun_Type *) wherefirstmax_float},
+   {SLANG_DOUBLE_TYPE, SLANG_DOUBLE_TYPE, SLANG_ARRAY_INDEX_TYPE, (SLarray_Contract_Fun_Type *) wherefirstmax_double},
+#endif
+     {0, 0, 0, NULL}
+};
+
+static void
+array_wherefirstmax (void)
+{
+   DO_WHEREXXX_YYY ("wherefirstmax", Array_WhereFirstMax_Funs);
+}
+
+static SLCONST SLarray_Contract_Type Array_WhereLastMin_Funs [] =
+{
+   {SLANG_CHAR_TYPE, SLANG_CHAR_TYPE, SLANG_ARRAY_INDEX_TYPE, (SLarray_Contract_Fun_Type *) wherelastmin_char},
+   {SLANG_UCHAR_TYPE, SLANG_UCHAR_TYPE, SLANG_ARRAY_INDEX_TYPE, (SLarray_Contract_Fun_Type *) wherelastmin_uchar},
+   {SLANG_SHORT_TYPE, SLANG_SHORT_TYPE, SLANG_ARRAY_INDEX_TYPE, (SLarray_Contract_Fun_Type *) wherelastmin_short},
+   {SLANG_USHORT_TYPE, SLANG_USHORT_TYPE, SLANG_ARRAY_INDEX_TYPE, (SLarray_Contract_Fun_Type *) wherelastmin_ushort},
+   {SLANG_INT_TYPE, SLANG_INT_TYPE, SLANG_ARRAY_INDEX_TYPE, (SLarray_Contract_Fun_Type *) wherelastmin_int},
+   {SLANG_UINT_TYPE, SLANG_UINT_TYPE, SLANG_ARRAY_INDEX_TYPE, (SLarray_Contract_Fun_Type *) wherelastmin_uint},
+   {SLANG_LONG_TYPE, SLANG_LONG_TYPE, SLANG_ARRAY_INDEX_TYPE, (SLarray_Contract_Fun_Type *) wherelastmin_long},
+   {SLANG_ULONG_TYPE, SLANG_ULONG_TYPE, SLANG_ARRAY_INDEX_TYPE, (SLarray_Contract_Fun_Type *) wherelastmin_ulong},
+#if defined(HAVE_LONG_LONG) && (SIZEOF_LONG_LONG != SIZEOF_LONG)
+   {SLANG_LLONG_TYPE, SLANG_LLONG_TYPE, SLANG_ARRAY_INDEX_TYPE, (SLarray_Contract_Fun_Type *) wherelastmin_llong},
+   {SLANG_ULLONG_TYPE, SLANG_ULLONG_TYPE, SLANG_ARRAY_INDEX_TYPE, (SLarray_Contract_Fun_Type *) wherelastmin_ullong},
+#endif
+#if SLANG_HAS_FLOAT
+   {SLANG_FLOAT_TYPE, SLANG_FLOAT_TYPE, SLANG_ARRAY_INDEX_TYPE, (SLarray_Contract_Fun_Type *) wherelastmin_float},
+   {SLANG_DOUBLE_TYPE, SLANG_DOUBLE_TYPE, SLANG_ARRAY_INDEX_TYPE, (SLarray_Contract_Fun_Type *) wherelastmin_double},
+#endif
+     {0, 0, 0, NULL}
+};
+
+static void
+array_wherelastmin (void)
+{
+   DO_WHEREXXX_YYY ("wherelastmin", Array_WhereLastMin_Funs);
+}
+
+static SLCONST SLarray_Contract_Type Array_WhereLastMax_Funs [] =
+{
+   {SLANG_CHAR_TYPE, SLANG_CHAR_TYPE, SLANG_ARRAY_INDEX_TYPE, (SLarray_Contract_Fun_Type *) wherelastmax_char},
+   {SLANG_UCHAR_TYPE, SLANG_UCHAR_TYPE, SLANG_ARRAY_INDEX_TYPE, (SLarray_Contract_Fun_Type *) wherelastmax_uchar},
+   {SLANG_SHORT_TYPE, SLANG_SHORT_TYPE, SLANG_ARRAY_INDEX_TYPE, (SLarray_Contract_Fun_Type *) wherelastmax_short},
+   {SLANG_USHORT_TYPE, SLANG_USHORT_TYPE, SLANG_ARRAY_INDEX_TYPE, (SLarray_Contract_Fun_Type *) wherelastmax_ushort},
+   {SLANG_INT_TYPE, SLANG_INT_TYPE, SLANG_ARRAY_INDEX_TYPE, (SLarray_Contract_Fun_Type *) wherelastmax_int},
+   {SLANG_UINT_TYPE, SLANG_UINT_TYPE, SLANG_ARRAY_INDEX_TYPE, (SLarray_Contract_Fun_Type *) wherelastmax_uint},
+   {SLANG_LONG_TYPE, SLANG_LONG_TYPE, SLANG_ARRAY_INDEX_TYPE, (SLarray_Contract_Fun_Type *) wherelastmax_long},
+   {SLANG_ULONG_TYPE, SLANG_ULONG_TYPE, SLANG_ARRAY_INDEX_TYPE, (SLarray_Contract_Fun_Type *) wherelastmax_ulong},
+#if defined(HAVE_LONG_LONG) && (SIZEOF_LONG_LONG != SIZEOF_LONG)
+   {SLANG_LLONG_TYPE, SLANG_LLONG_TYPE, SLANG_ARRAY_INDEX_TYPE, (SLarray_Contract_Fun_Type *) wherelastmax_llong},
+   {SLANG_ULLONG_TYPE, SLANG_ULLONG_TYPE, SLANG_ARRAY_INDEX_TYPE, (SLarray_Contract_Fun_Type *) wherelastmax_ullong},
+#endif
+#if SLANG_HAS_FLOAT
+   {SLANG_FLOAT_TYPE, SLANG_FLOAT_TYPE, SLANG_ARRAY_INDEX_TYPE, (SLarray_Contract_Fun_Type *) wherelastmax_float},
+   {SLANG_DOUBLE_TYPE, SLANG_DOUBLE_TYPE, SLANG_ARRAY_INDEX_TYPE, (SLarray_Contract_Fun_Type *) wherelastmax_double},
+#endif
+     {0, 0, 0, NULL}
+};
+
+static void
+array_wherelastmax (void)
+{
+   DO_WHEREXXX_YYY ("wherelastmax", Array_WhereLastMax_Funs);
+}
+
+
 static SLCONST SLarray_Contract_Type Array_Maxabs_Funs [] =
 {
      {SLANG_CHAR_TYPE, SLANG_CHAR_TYPE, SLANG_CHAR_TYPE, (SLarray_Contract_Fun_Type *) maxabs_chars},
@@ -1285,7 +1541,6 @@ static void array_swap (void)
    int len;
    unsigned char *src, *dst;
    size_t sizeof_type;
-   unsigned int k;
    int dim, have_dim;
    SLang_Array_Type *at;
 #if 0
@@ -1329,6 +1584,8 @@ static void array_swap (void)
    sizeof_type = at->cl->cl_sizeof_type;
    if (have_dim == 0)
      {
+	unsigned int k;
+
 	src = (unsigned char *)at->data + j*sizeof_type;
 	dst = (unsigned char *)at->data + i*sizeof_type;
 
@@ -1506,8 +1763,8 @@ static SLCONST SLarray_Contract_Type Array_Any_Funs [] =
      {SLANG_LONG_TYPE, SLANG_LONG_TYPE, SLANG_CHAR_TYPE, (SLarray_Contract_Fun_Type *) any_longs},
      {SLANG_ULONG_TYPE, SLANG_ULONG_TYPE, SLANG_CHAR_TYPE, (SLarray_Contract_Fun_Type *) any_ulongs},
 #if defined(HAVE_LONG_LONG) && (SIZEOF_LONG_LONG != SIZEOF_LONG)
-     {SLANG_LLONG_TYPE, SLANG_LLONG_TYPE, SLANG_LLONG_TYPE, (SLarray_Contract_Fun_Type *) any_llongs},
-     {SLANG_ULLONG_TYPE, SLANG_ULLONG_TYPE, SLANG_ULLONG_TYPE, (SLarray_Contract_Fun_Type *) any_ullongs},
+     {SLANG_LLONG_TYPE, SLANG_LLONG_TYPE, SLANG_CHAR_TYPE, (SLarray_Contract_Fun_Type *) any_llongs},
+     {SLANG_ULLONG_TYPE, SLANG_ULLONG_TYPE, SLANG_CHAR_TYPE, (SLarray_Contract_Fun_Type *) any_ullongs},
 #endif
 #if SLANG_HAS_FLOAT
      {SLANG_FLOAT_TYPE, SLANG_FLOAT_TYPE, SLANG_CHAR_TYPE, (SLarray_Contract_Fun_Type *) any_floats},
@@ -1533,8 +1790,8 @@ static SLCONST SLarray_Contract_Type Array_All_Funs [] =
      {SLANG_LONG_TYPE, SLANG_LONG_TYPE, SLANG_CHAR_TYPE, (SLarray_Contract_Fun_Type *) all_longs},
      {SLANG_ULONG_TYPE, SLANG_ULONG_TYPE, SLANG_CHAR_TYPE, (SLarray_Contract_Fun_Type *) all_ulongs},
 #if defined(HAVE_LONG_LONG) && (SIZEOF_LONG_LONG != SIZEOF_LONG)
-     {SLANG_LLONG_TYPE, SLANG_LLONG_TYPE, SLANG_LLONG_TYPE, (SLarray_Contract_Fun_Type *) all_llongs},
-     {SLANG_ULLONG_TYPE, SLANG_ULLONG_TYPE, SLANG_ULLONG_TYPE, (SLarray_Contract_Fun_Type *) all_ullongs},
+     {SLANG_LLONG_TYPE, SLANG_LLONG_TYPE, SLANG_CHAR_TYPE, (SLarray_Contract_Fun_Type *) all_llongs},
+     {SLANG_ULLONG_TYPE, SLANG_ULLONG_TYPE, SLANG_CHAR_TYPE, (SLarray_Contract_Fun_Type *) all_ullongs},
 #endif
 #if SLANG_HAS_FLOAT
      {SLANG_FLOAT_TYPE, SLANG_FLOAT_TYPE, SLANG_CHAR_TYPE, (SLarray_Contract_Fun_Type *) all_floats},
@@ -1563,6 +1820,199 @@ static void set_innerprod_block_size (int *sp)
    Inner_Prod_Block_Size = (unsigned int) s;
 }
 
+static int do_wherefirstlast_op (const char *fname, int is_first, int op)
+{
+   SLindex_Type istart;
+   SLang_Array_Type *at;
+   SLindex_Type num;
+   int use_double, type, status;
+   double b;
+
+   istart = (is_first ? 0 : -1);
+
+   switch (SLang_Num_Function_Args)
+     {
+      case 3:
+	if (-1 == SLang_pop_array_index (&istart))
+	  return -1;
+	/* drop */
+      case 2:
+	if (-1 == SLstack_exch (0, 1))
+	  return -1;
+	break;
+
+      default:
+	_pSLang_verror (SL_Usage_Error, "Usage: i = %s (array, double [,istart]", fname);
+	return -1;
+     }
+
+   if (-1 == SLang_pop_array (&at, 1))
+     return -1;
+
+   status = -1;
+
+   num = at->num_elements;
+   if (istart < 0)
+     istart += num;
+   if (istart < 0)
+     {
+	if (num == 0)
+	  istart = 0;
+	else
+	  {
+	     SLang_set_error (SL_Index_Error);
+	     goto free_and_return;
+	  }
+     }
+   else if (istart > num)
+     istart = num;
+
+   use_double = 1;
+   type = at->data_type;
+
+#if _pSLANG_INT64_TYPE
+   if ((type == _pSLANG_INT64_TYPE) || (type == _pSLANG_UINT64_TYPE))
+     {
+	int type1;
+	if (-1 == (type1 = SLang_peek_at_stack ()))
+	  goto free_and_return;
+
+	/* Use the following as a proxy for determining an integral type */
+	use_double = (0 == SLang_get_int_size ((SLtype) type1));
+     }
+#endif
+
+   if (use_double
+       && (-1 == SLang_pop_double (&b)))
+     goto free_and_return;
+
+   switch (type)
+     {
+      case SLANG_CHAR_TYPE:
+	if (is_first)
+	  status = wherefirst_op_char (at, op, b, istart);
+	else
+	  status = wherelast_op_char (at, op, b, istart);
+	break;
+
+      case SLANG_UCHAR_TYPE:
+	if (is_first)
+	  status = wherefirst_op_uchar (at, op, b, istart);
+	else
+	  status = wherelast_op_uchar (at, op, b, istart);
+	break;
+
+      case _pSLANG_INT16_TYPE:
+	if (is_first)
+	  status = wherefirst_op_int16 (at, op, b, istart);
+	else
+	  status = wherelast_op_int16 (at, op, b, istart);
+	break;
+
+      case _pSLANG_UINT16_TYPE:
+	if (is_first)
+	  status = wherefirst_op_uint16 (at, op, b, istart);
+	else
+	  status = wherelast_op_uint16 (at, op, b, istart);
+	break;
+
+      case _pSLANG_INT32_TYPE:
+	if (is_first)
+	  status = wherefirst_op_int32 (at, op, b, istart);
+	else
+	  status = wherelast_op_int32 (at, op, b, istart);
+	break;
+
+      case _pSLANG_UINT32_TYPE:
+	if (is_first)
+	  status = wherefirst_op_uint32 (at, op, b, istart);
+	else
+	  status = wherelast_op_uint32 (at, op, b, istart);
+	break;
+
+#if _pSLANG_INT64_TYPE
+      case _pSLANG_INT64_TYPE:
+	if (use_double)
+	  {
+	     if (is_first)
+	       status = wherefirst_op_int64 (at, op, b, istart);
+	     else
+	       status = wherelast_op_int64 (at, op, b, istart);
+	  }
+	else
+	  {
+	     _pSLint64_Type b64;
+	     if (-1 == (status = _pSLang_pop_int64 (&b64)))
+	       break;
+	     if (is_first)
+	       status = wherefirst_op_int64_alt (at, op, b64, istart);
+	     else
+	       status = wherelast_op_int64_alt (at, op, b64, istart);
+	  }
+	break;
+
+      case _pSLANG_UINT64_TYPE:
+	if (use_double)
+	  {
+	     if (is_first)
+	       status = wherefirst_op_uint64 (at, op, b, istart);
+	     else
+	       status = wherelast_op_uint64 (at, op, b, istart);
+	  }
+	else
+	  {
+	     _pSLuint64_Type b64;
+	     if (-1 == (status = _pSLang_pop_uint64 (&b64)))
+	       break;
+	     if (is_first)
+	       status = wherefirst_op_uint64_alt (at, op, b64, istart);
+	     else
+	       status = wherelast_op_uint64_alt (at, op, b64, istart);
+	  }
+	break;
+#endif				       /* _pSLANG_INT64_TYPE */
+
+      case SLANG_FLOAT_TYPE:
+	if (is_first)
+	  status = wherefirst_op_float (at, op, b, istart);
+	else
+	  status = wherelast_op_float (at, op, b, istart);
+	break;
+
+      case SLANG_DOUBLE_TYPE:
+	if (is_first)
+	  status = wherefirst_op_double (at, op, b, istart);
+	else
+	  status = wherelast_op_double (at, op, b, istart);
+	break;
+
+      default:
+	SLang_verror (SL_NotImplemented_Error, "%s: support for arrays of type %s is not available",
+		      fname, SLclass_get_datatype_name (type));
+	goto free_and_return;
+     }
+
+free_and_return:
+
+   SLang_free_array (at);
+   return status;
+}
+
+
+static void wherefirsteq(void) {(void) do_wherefirstlast_op ("wherefirsteq", 1, SLANG_EQ);}
+static void wherefirstne(void) {(void) do_wherefirstlast_op ("wherefirstne", 1, SLANG_NE);}
+static void wherefirstlt(void) {(void) do_wherefirstlast_op ("wherefirstlt", 1, SLANG_LT);}
+static void wherefirstgt(void) {(void) do_wherefirstlast_op ("wherefirstgt", 1, SLANG_GT);}
+static void wherefirstle(void) {(void) do_wherefirstlast_op ("wherefirstle", 1, SLANG_LE);}
+static void wherefirstge(void) {(void) do_wherefirstlast_op ("wherefirstge", 1, SLANG_GE);}
+
+static void wherelasteq(void) {(void) do_wherefirstlast_op ("wherelasteq", 0, SLANG_EQ);}
+static void wherelastne(void) {(void) do_wherefirstlast_op ("wherelastne", 0, SLANG_NE);}
+static void wherelastlt(void) {(void) do_wherefirstlast_op ("wherelastlt", 0, SLANG_LT);}
+static void wherelastgt(void) {(void) do_wherefirstlast_op ("wherelastgt", 0, SLANG_GT);}
+static void wherelastle(void) {(void) do_wherefirstlast_op ("wherelastle", 0, SLANG_LE);}
+static void wherelastge(void) {(void) do_wherefirstlast_op ("wherelastge", 0, SLANG_GE);}
+
 static SLang_Intrin_Fun_Type Array_Fun_Table [] =
 {
    MAKE_INTRINSIC_1("transpose", array_transpose, SLANG_VOID_TYPE, SLANG_ARRAY_TYPE),
@@ -1581,6 +2031,25 @@ static SLang_Intrin_Fun_Type Array_Fun_Table [] =
    MAKE_INTRINSIC_0("any", array_any, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_0("all", array_all, SLANG_VOID_TYPE),
 
+   MAKE_INTRINSIC_0("wherefirstmin", array_wherefirstmin, SLANG_VOID_TYPE),
+   MAKE_INTRINSIC_0("wherefirstmax", array_wherefirstmax, SLANG_VOID_TYPE),
+   MAKE_INTRINSIC_0("wherelastmin", array_wherelastmin, SLANG_VOID_TYPE),
+   MAKE_INTRINSIC_0("wherelastmax", array_wherelastmax, SLANG_VOID_TYPE),
+
+   MAKE_INTRINSIC_0("wherefirst_eq", wherefirsteq, SLANG_VOID_TYPE),
+   MAKE_INTRINSIC_0("wherefirst_ne", wherefirstne, SLANG_VOID_TYPE),
+   MAKE_INTRINSIC_0("wherefirst_gt", wherefirstgt, SLANG_VOID_TYPE),
+   MAKE_INTRINSIC_0("wherefirst_lt", wherefirstlt, SLANG_VOID_TYPE),
+   MAKE_INTRINSIC_0("wherefirst_ge", wherefirstge, SLANG_VOID_TYPE),
+   MAKE_INTRINSIC_0("wherefirst_le", wherefirstle, SLANG_VOID_TYPE),
+
+   MAKE_INTRINSIC_0("wherelast_eq", wherelasteq, SLANG_VOID_TYPE),
+   MAKE_INTRINSIC_0("wherelast_ne", wherelastne, SLANG_VOID_TYPE),
+   MAKE_INTRINSIC_0("wherelast_gt", wherelastgt, SLANG_VOID_TYPE),
+   MAKE_INTRINSIC_0("wherelast_lt", wherelastlt, SLANG_VOID_TYPE),
+   MAKE_INTRINSIC_0("wherelast_ge", wherelastge, SLANG_VOID_TYPE),
+   MAKE_INTRINSIC_0("wherelast_le", wherelastle, SLANG_VOID_TYPE),
+
    MAKE_INTRINSIC_0("__get_innerprod_block_size", get_innerprod_block_size, SLANG_INT_TYPE),
    MAKE_INTRINSIC_I("__set_innerprod_block_size", set_innerprod_block_size, SLANG_VOID_TYPE),
 
