@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2004-2010 John E. Davis
+Copyright (C) 2004-2017,2018 John E. Davis
 
 This file is part of the S-Lang Library.
 
@@ -89,21 +89,69 @@ USA.
 /* Colors:  These definitions are used for the display.  However, the
  * application only uses object handles which get mapped to this
  * internal representation.  The mapping is performed by the Color_Map
- * structure below. */
-
-#define CHAR_MASK	0x000000FF
-#define FG_MASK		0x0000FF00
-#define BG_MASK		0x00FF0000
-#define ATTR_MASK	0x1F000000
-#define BGALL_MASK	0x0FFF0000
-
-/* The 0x10000000 bit represents the alternate character set.  BGALL_MASK does
- * not include this attribute.
+ * structure below.
+ *
+ * Colors are encoded in 25 bits as follows:
+ * - Values 0-255 for standard palette
+ * - 256 for terminal's default color
+ * - 0x1RRGGBB for true colors
+ *
+ * The bits are split so we can maintain ABI compatibility on 64 bit machines.
  */
+#if SLTT_HAS_TRUECOLOR_SUPPORT && (SIZEOF_LONG == 8)
+static int Has_True_Color = 0;
+# if 0
+#  define FG_MASK_LOW		0x0000000000FFFFFFULL
+#  define FG_MASK_HIGH		0x0200000000000000ULL
+#  define BG_MASK		0x01FFFFFF00000000ULL
+#  define ATTR_MASK		0x000000003F000000ULL
+#  define INVALID_ATTR		0xFFFFFFFFFFFFFFFFULL
+#  define SLSMG_COLOR_DEFAULT	0x0000000000000100ULL
 
-#define GET_FG(fgbg) (((fgbg) & FG_MASK) >> 8)
-#define GET_BG(fgbg) (((fgbg) & BG_MASK) >> 16)
-#define MAKE_COLOR(fg, bg) (((fg) | ((bg) << 8)) << 8)
+#  define TRUE_COLOR_BIT		0x01000000ULL
+#  define IS_TRUE_COLOR(f)	((f) & TRUE_COLOR_BIT)
+#  define GET_FG(fgbg) ((((fgbg) & FG_MASK_HIGH) >> 33) | ((fgbg) & FG_MASK_LOW))
+#  define GET_BG(fgbg) (((fgbg) & BG_MASK) >> 32)
+#  define MAKE_COLOR(f, b) \
+   ((((f) << 33) & FG_MASK_HIGH) | ((f) & FG_MASK_LOW) \
+     | (((b) << 32) & BG_MASK))
+# else
+#  define FG_MASK_LOW		0x000000000000FF00ULL
+#  define FG_MASK_HIGH		0x0000FFFF00000000ULL
+#  define FG_TRUE_COLOR		0x0000000000000001ULL
+#  define BG_MASK_LOW		0x0000000000FF0000ULL
+#  define BG_MASK_HIGH		0xFFFF000000000000ULL
+#  define BG_TRUE_COLOR		0x0000000000000002ULL
+#  define ATTR_MASK		0x000000003F000000ULL
+#  define INVALID_ATTR		0xFFFFFFFFFFFFFFFFULL
+
+#  define SLSMG_COLOR_DEFAULT		0x00000100ULL
+#  define TRUE_COLOR_BIT		0x01000000ULL
+#  define IS_TRUE_COLOR(f)	((f) & TRUE_COLOR_BIT)
+#  define GET_FG(fgbg) \
+   ((((fgbg)&FG_TRUE_COLOR)<<24) | (((fgbg)&FG_MASK_HIGH)>>24) \
+     | (((fgbg)&FG_MASK_LOW)>>8))
+#  define GET_BG(fgbg) \
+   ((((fgbg)&BG_TRUE_COLOR)<<23) | (((fgbg)&BG_MASK_HIGH)>>40) \
+     | (((fgbg)&BG_MASK_LOW)>>16))
+#  define MAKE_COLOR(f, b) \
+   (((((f)&0x1000000)>>24) | (((f)&0xFF)<<8) | (((f)&(0xFFFF00))<<24)) \
+     | ((((b)&0x1000000)>>23) | (((b)&0xFF)<<16) | (((b)&(0xFFFF00))<<40)))
+# endif
+#else
+# undef SLTT_HAS_TRUECOLOR_SUPPORT
+# define SLTT_HAS_TRUECOLOR_SUPPORT 0
+# define FG_MASK_LOW		0x0000FF00UL
+# define FG_MASK_HIGH		0x00000000UL
+# define BG_MASK		0x00FF0000UL
+# define ATTR_MASK		0x3F000000UL
+# define INVALID_ATTR		0xFFFFFFFFUL
+# define SLSMG_COLOR_DEFAULT	0x000000FFUL
+# define IS_TRUE_COLOR(f)	(0)
+# define GET_FG(fgbg) (((fgbg) & FG_MASK_LOW)>>8)
+# define GET_BG(fgbg) (((fgbg) & BG_MASK) >> 16)
+# define MAKE_COLOR(fg, bg) (((fg)<<8) | ((bg)<<16))
+#endif
 
 int SLtt_Screen_Cols = 80;
 int SLtt_Screen_Rows = 24;
@@ -157,7 +205,11 @@ static int Mouse_Mode = -1;
  * the highbit is set, we interpret that as a blink character.  This is
  * exploited by DOSemu.
  */
-#define JMAX_COLORS 512
+#ifndef SLTT_MAX_COLORS
+# define SLTT_MAX_COLORS 0x8000       /* consistent with SLSMG_COLOR_MASK */
+#endif
+
+#define JMAX_COLORS SLTT_MAX_COLORS
 #define JNORMAL_COLOR 0
 
 typedef struct
@@ -180,6 +232,10 @@ static SLCONST int RGB_to_BGR[] =
 
 static SLCONST char *Color_Fg_Str = "\033[3%dm";
 static SLCONST char *Color_Bg_Str = "\033[4%dm";
+#if SLTT_HAS_TRUECOLOR_SUPPORT
+static SLCONST char *Color_RGB_Fg_Str = "\033[38;2;%d;%d;%dm";
+static SLCONST char *Color_RGB_Bg_Str = "\033[48;2;%d;%d;%dm";
+#endif
 static SLCONST char *Default_Color_Fg_Str = "\033[39m";
 static SLCONST char *Default_Color_Bg_Str = "\033[49m";
 
@@ -192,6 +248,7 @@ char *SLtt_Graphics_Char_Pairs = NULL;	       /* ac termcap string -- def is vt1
 
 static SLCONST char *UnderLine_Vid_Str;
 static SLCONST char *Blink_Vid_Str;
+static SLCONST char *Italic_Vid_Str;
 static SLCONST char *Bold_Vid_Str;
 static SLCONST char *Ins_Mode_Str; /* = "\033[4h"; */   /* ins mode (im) */
 static SLCONST char *Eins_Mode_Str; /* = "\033[4l"; */  /* end ins mode (ei) */
@@ -246,7 +303,7 @@ static int Scroll_r1 = 0, Scroll_r2 = 23;
 static int Cursor_r, Cursor_c;	       /* 0 based */
 
 /* current attributes --- initialized to impossible value */
-static SLtt_Char_Type Current_Fgbg = 0xFFFFFFFFU;
+static SLtt_Char_Type Current_Fgbg = INVALID_ATTR;
 
 static int Cursor_Set;		       /* 1 if cursor position known, 0
 					* if not.  -1 if only row is known
@@ -275,16 +332,15 @@ int _pSLusleep (unsigned long usecs)
 
 int SLtt_flush_output (void)
 {
-   int nwrite = 0;
-   unsigned int total;
-   int n = (int) (Output_Bufferp - Output_Buffer);
+   size_t total;
+   size_t n = (Output_Bufferp - Output_Buffer);
 
    SLtt_Num_Chars_Output += n;
 
    total = 0;
    while (n > 0)
      {
-	nwrite = write (SLang_TT_Write_FD, (char *) Output_Buffer + total, n);
+	ssize_t nwrite = write (SLang_TT_Write_FD, (char *) Output_Buffer + total, n);
 	if (nwrite == -1)
 	  {
 	     nwrite = 0;
@@ -315,22 +371,20 @@ int SLtt_flush_output (void)
 }
 
 int SLtt_Baud_Rate = 0;
-static void tt_write(SLCONST char *str, unsigned int n)
+static void tt_write(SLCONST char *str, SLstrlen_Type n)
 {
    static unsigned long last_time;
-   static int total;
-   unsigned long now;
-   unsigned int ndiff;
+   static SLstrlen_Type total;
 
    if ((str == NULL) || (n == 0)) return;
    total += n;
 
    while (1)
      {
-	ndiff = MAX_OUTPUT_BUFFER_SIZE - (int) (Output_Bufferp - Output_Buffer);
+	size_t ndiff = MAX_OUTPUT_BUFFER_SIZE - (Output_Bufferp - Output_Buffer);
 	if (ndiff < n)
 	  {
-	     SLMEMCPY ((char *) Output_Bufferp, str, ndiff);
+	     memcpy ((char *) Output_Bufferp, str, ndiff);
 	     Output_Bufferp += ndiff;
 	     SLtt_flush_output ();
 	     n -= ndiff;
@@ -338,15 +392,16 @@ static void tt_write(SLCONST char *str, unsigned int n)
 	  }
 	else
 	  {
-	     SLMEMCPY ((char *) Output_Bufferp, str, n);
+	     memcpy ((char *) Output_Bufferp, str, n);
 	     Output_Bufferp += n;
 	     break;
 	  }
      }
 
    if (((SLtt_Baud_Rate > 150) && (SLtt_Baud_Rate <= 9600))
-       && (10 * total > SLtt_Baud_Rate))
+       && (10 * total > (unsigned int)SLtt_Baud_Rate))
      {
+	unsigned long now;
 	total = 0;
 	if ((now = (unsigned long) time(NULL)) - last_time <= 1)
 	  {
@@ -389,10 +444,13 @@ void SLtt_putchar (char ch)
    else tt_write (&ch, 1);
 }
 
+#if defined(__GNUC__)
+# pragma GCC diagnostic ignored "-Wformat-nonliteral"
+#endif
 static unsigned int tt_sprintf(char *buf, unsigned int buflen, SLCONST char *fmt, int x, int y)
 {
    SLCONST char *fmt_max;
-   unsigned char *b, *bmax, ch;
+   unsigned char *b, *bmax;
    int offset;
    int z, z1, parse_level;
    int zero_pad;
@@ -427,7 +485,7 @@ static unsigned int tt_sprintf(char *buf, unsigned int buflen, SLCONST char *fmt
 
    while ((fmt < fmt_max) && (b < bmax))
      {
-	ch = *fmt++;
+	unsigned char ch = *fmt++;
 
 	if (ch != '%')
 	  {
@@ -689,6 +747,9 @@ static unsigned int tt_sprintf(char *buf, unsigned int buflen, SLCONST char *fmt
 
    return (unsigned int) (b - (unsigned char *) buf);
 }
+#if defined(__GNUC__)
+# pragma GCC diagnostic warning "-Wformat-nonliteral"
+#endif
 
 static void tt_printf(SLCONST char *fmt, int x, int y)
 {
@@ -792,7 +853,6 @@ static void goto_relative_rc (int r, int c)
 void SLtt_goto_rc(int r, int c)
 {
    char *s = NULL;
-   int n;
    char buf[6];
 
    if ((c < 0) || (r < 0))
@@ -814,7 +874,7 @@ void SLtt_goto_rc(int r, int c)
    if ((Cursor_Set > 0)
        || ((Cursor_Set < 0) && !Automatic_Margins))
      {
-	n = r - Cursor_r;
+	int n = r - Cursor_r;
 	if ((n == -1) && (Cursor_Set > 0) && (Cursor_c == c)
 	    && (Curs_Up_Str != NULL))
 	  {
@@ -907,7 +967,6 @@ static void delete_line_in_scroll_region (void)
 
 void SLtt_delete_nlines (int nn)
 {
-   int r1, curs;
    unsigned int n;
 
    if (nn <= 0) return;
@@ -926,6 +985,7 @@ void SLtt_delete_nlines (int nn)
    /* get a new terminal */
      {
 	char buf[80];
+	int curs, r1;
 	unsigned int dn = n;
 
 	if (dn > sizeof (buf))
@@ -1044,7 +1104,7 @@ static void del_eol (void)
      }
 
    if ((Del_Eol_Str != NULL)
-       && (Can_Background_Color_Erase || ((Current_Fgbg & ~0xFF) == 0)))
+       && (Can_Background_Color_Erase || (Current_Fgbg == 0)))
      {
 	tt_write_string(Del_Eol_Str);
 	return;
@@ -1061,7 +1121,7 @@ static void del_eol (void)
 
 void SLtt_del_eol (void)
 {
-   if (Current_Fgbg != 0xFFFFFFFFU) SLtt_normal_video ();
+   if (Current_Fgbg != INVALID_ATTR) SLtt_normal_video ();
    del_eol ();
 }
 
@@ -1088,19 +1148,18 @@ static Color_Def_Type Color_Defs [MAX_COLOR_NAMES] =
      {"brightgreen",	SLSMG_COLOR_BRIGHT_GREEN},
      {"yellow",		SLSMG_COLOR_BRIGHT_BROWN},
      {"brightblue",	SLSMG_COLOR_BRIGHT_BLUE},
-     {"brightmagenta",	SLSMG_COLOR_BRIGHT_CYAN},
-     {"brightcyan",	SLSMG_COLOR_BRIGHT_MAGENTA},
+     {"brightmagenta",	SLSMG_COLOR_BRIGHT_MAGENTA},
+     {"brightcyan",	SLSMG_COLOR_BRIGHT_CYAN},
      {"white",		SLSMG_COLOR_BRIGHT_WHITE},
-#define SLSMG_COLOR_DEFAULT 0xFF
-     {"default",		SLSMG_COLOR_DEFAULT}
+     {"default",	SLSMG_COLOR_DEFAULT}
 };
 
 static int Brushes_Initialized = 0;
 
 static int initialize_brushes (void)
 {
-   int fg, bg;
    Brush_Info_Type *b, *bmax;
+   SLtt_Char_Type bg;
 
    b = Brush_Table;
    bmax = b + JMAX_COLORS;
@@ -1108,7 +1167,7 @@ static int initialize_brushes (void)
    bg = 0;
    while (b < bmax)
      {
-	fg = 7;
+	SLtt_Char_Type fg = 7;
 	while (b < bmax)
 	  {
 	     if (fg != bg)
@@ -1149,7 +1208,7 @@ static SLtt_Char_Type get_brush_attr (SLsmg_Color_Type color)
    Brush_Info_Type *b;
 
    if (NULL == (b = get_brush_info (color)))
-     return (SLtt_Char_Type)-1;
+     return INVALID_ATTR;
 
    if (SLtt_Use_Ansi_Colors)
      return b->fgbg;
@@ -1159,7 +1218,10 @@ static SLtt_Char_Type get_brush_attr (SLsmg_Color_Type color)
 
 static SLtt_Char_Type get_brush_fgbg (SLsmg_Color_Type color)
 {
-   return get_brush_info(color)->fgbg;
+   Brush_Info_Type *b = get_brush_info(color);
+   if (b == NULL)
+     return INVALID_ATTR;
+   return b->fgbg;
 }
 
 int SLtt_set_mono (int obj, SLFUTURE_CONST char *what, SLtt_Char_Type mask)
@@ -1304,11 +1366,17 @@ static SLtt_Char_Type fb_to_fgbg (SLtt_Char_Type f, SLtt_Char_Type b)
 {
    SLtt_Char_Type attr;
 
-   if (Max_Terminfo_Colors != 8)
+   if ((Max_Terminfo_Colors != 8)
+#if SLTT_HAS_TRUECOLOR_SUPPORT
+       || Has_True_Color
+#endif
+      )
      {
-	if (f != SLSMG_COLOR_DEFAULT) f %= Max_Terminfo_Colors;
-	if (b != SLSMG_COLOR_DEFAULT) b %= Max_Terminfo_Colors;
-	return ((f << 8) | (b << 16));
+	if ((f != SLSMG_COLOR_DEFAULT) && (0 == IS_TRUE_COLOR(f)))
+	  f %= Max_Terminfo_Colors;
+	if ((b != SLSMG_COLOR_DEFAULT) && (0 == IS_TRUE_COLOR(b)))
+	  b %= Max_Terminfo_Colors;
+	return MAKE_COLOR(f,b);
      }
 
    /* Otherwise we have 8 ansi colors.  Try to get bright versions
@@ -1330,16 +1398,57 @@ static SLtt_Char_Type fb_to_fgbg (SLtt_Char_Type f, SLtt_Char_Type b)
 	b &= 0x7;
      }
 
-   return ((f << 8) | (b << 16) | attr);
+   return MAKE_COLOR(f,b) | attr;
 }
 
+#if SLTT_HAS_TRUECOLOR_SUPPORT
+static int parse_hex_digit (char ch)
+{
+   if (('0' <= ch) && (ch <= '9')) return ch - '0';
+   if (('A' <= ch) && (ch <= 'F')) return 10 + ch - 'A';
+   if (('a' <= ch) && (ch <= 'f')) return 10 + ch - 'a';
+   return -1;
+}
+
+static int parse_true_color (const char *color, SLtt_Char_Type *c)
+{
+   SLtt_Char_Type rgb;
+   unsigned int i;
+   int h[6];
+
+   i = 0;
+   while (i < 6)
+     {
+	if (-1 == (h[i] = parse_hex_digit (color[i])))
+	  return -1;
+	i++;
+     }
+   if (color[i] != 0)
+     return -1;
+
+   if (i == 3)			       /* RRGGBB */
+     rgb = (h[0] << 20) | (h[0] << 16) | (h[1] << 12) | (h[1] << 8) | (h[2] << 4) | h[2];
+   else if ((i == 6) && (color[i] == 0))
+     rgb = (h[0] << 20) | (h[1] << 16) | (h[2] << 12) | (h[3] << 8) | (h[4] << 4) | h[5];
+   else
+     return -1;
+
+   *c = rgb | TRUE_COLOR_BIT;
+   return 0;
+}
+#endif
+
 /* This looks for colors with name form 'colorN'.  If color is of this
  * form, N is passed back via parameter list.
  */
 static int parse_color_digit_name (SLCONST char *color, SLtt_Char_Type *f)
 {
    unsigned int i;
-   unsigned char ch;
+
+#if SLTT_HAS_TRUECOLOR_SUPPORT
+   if (Has_True_Color && (color[0] == '#'))
+     return parse_true_color (color+1, f);
+#endif
 
    if (strncmp (color, "color", 5))
      return -1;
@@ -1352,6 +1461,7 @@ static int parse_color_digit_name (SLCONST char *color, SLtt_Char_Type *f)
    while (1)
      {
 	unsigned int j;
+	unsigned char ch;
 
 	ch = (unsigned char) *color++;
 	if (ch == 0)
@@ -1371,11 +1481,59 @@ static int parse_color_digit_name (SLCONST char *color, SLtt_Char_Type *f)
    return 0;
 }
 
+/* Here whitespace is not allowed.  That is, "blue;blink" is ok but
+ * "blue; blink" or "blue ;blink" are not.
+ */
+static int parse_color_and_attributes (SLCONST char *f, char *buf, size_t buflen, SLtt_Char_Type *attrp)
+{
+   SLCONST char *s;
+   unsigned int len;
+   SLtt_Char_Type a;
+
+   *attrp = a = 0;
+
+   s = strchr (f, ';');
+   if (s == NULL) return 0;
+
+   len = s - f;
+   if (len >= buflen) len = buflen-1;
+   strncpy (buf, f, len);
+   buf[len] = 0;
+
+   while ((*s == ';') || (*s == ' ') || (*s == '\t')) s++;
+   f = s;
+   while (*f)
+     {
+	s = strchr (f, ';');
+	if (s == NULL)
+	  s = f + strlen (f);
+
+	len = s - f;
+	if (len)
+	  {
+	     if (0 == strncmp (f, "italic", 6))
+	       a |= SLTT_ITALIC_MASK;
+	     else if (0 == strncmp (f, "blink", 5))
+	       a |= SLTT_BLINK_MASK;
+	     else if (0 == strncmp (f, "underline", 9))
+	       a |= SLTT_ULINE_MASK;
+	     else if (0 == strncmp (f, "bold", 4))
+	       a |= SLTT_BOLD_MASK;
+	  }
+	while ((*s == ';') || (*s == ' ') || (*s == '\t')) s++;
+	f = s;
+     }
+   *attrp = a;
+   return 1;
+}
+
 static int make_color_fgbg (SLCONST char *fg, SLCONST char *bg, SLtt_Char_Type *fgbg)
 {
-   SLtt_Char_Type f = 0xFFFFFFFFU, b = 0xFFFFFFFFU;
+   SLtt_Char_Type f = INVALID_ATTR, b = INVALID_ATTR;
    SLCONST char *dfg, *dbg;
    unsigned int i;
+   char bgbuf[16], fgbuf[16];
+   SLtt_Char_Type fattr= 0, battr = 0;
 
    if ((fg != NULL) && (*fg == 0)) fg = NULL;
    if ((bg != NULL) && (*bg == 0)) bg = NULL;
@@ -1389,6 +1547,9 @@ static int make_color_fgbg (SLCONST char *fg, SLCONST char *bg, SLtt_Char_Type *
 	if (bg == NULL) bg = dbg;
      }
 
+   if (1 == parse_color_and_attributes (fg, fgbuf, sizeof(fgbuf), &fattr))
+     fg = fgbuf;
+
    if (-1 == parse_color_digit_name (fg, &f))
      {
 	for (i = 0; i < MAX_COLOR_NAMES; i++)
@@ -1399,6 +1560,9 @@ static int make_color_fgbg (SLCONST char *fg, SLCONST char *bg, SLtt_Char_Type *
 	  }
      }
 
+   if (1 == parse_color_and_attributes (bg, bgbuf, sizeof(bgbuf), &battr))
+     bg = bgbuf;
+
    if (-1 == parse_color_digit_name (bg, &b))
      {
 	for (i = 0; i < MAX_COLOR_NAMES; i++)
@@ -1409,10 +1573,10 @@ static int make_color_fgbg (SLCONST char *fg, SLCONST char *bg, SLtt_Char_Type *
 	  }
      }
 
-   if ((f == 0xFFFFFFFFU) || (b == 0xFFFFFFFFU))
+   if ((f == INVALID_ATTR) || (b == INVALID_ATTR))
      return -1;
 
-   *fgbg = fb_to_fgbg (f, b);
+   *fgbg = fb_to_fgbg (f, b) | fattr | battr;
    return 0;
 }
 
@@ -1450,9 +1614,29 @@ void SLtt_set_alt_char_set (int i)
    last_i = i;
 }
 
+#if SLTT_HAS_TRUECOLOR_SUPPORT
+# if defined(__GNUC__)
+#  pragma GCC diagnostic ignored "-Wformat-nonliteral"
+# endif
+static void write_truecolor (const char *fmt, SLtt_Char_Type c)
+{
+   char tmpbuf[32];
+   int r, g, b;
+
+   r = (int)((c>>16) & 0xFF);
+   g = (int)((c>>8) & 0xFF);
+   b = (int)(c & 0xFF);
+
+   sprintf (tmpbuf, fmt, r, g, b);
+   tt_write_string (tmpbuf);
+}
+# if defined(__GNUC__)
+#  pragma GCC diagnostic warning "-Wformat-nonliteral"
+# endif
+#endif				       /* SLTT_HAS_TRUECOLOR_SUPPORT */
+
 static void write_attributes (SLtt_Char_Type fgbg)
 {
-   int bg0, fg0;
    int unknown_attributes;
 
    if (Worthless_Highlight) return;
@@ -1481,6 +1665,7 @@ static void write_attributes (SLtt_Char_Type fgbg)
 	if (fgbg & SLTT_ULINE_MASK) tt_write_string (UnderLine_Vid_Str);
 	if (fgbg & SLTT_BOLD_MASK) SLtt_bold_video ();
 	if (fgbg & SLTT_REV_MASK) tt_write_string (Rev_Vid_Str);
+	if (fgbg & SLTT_ITALIC_MASK) tt_write_string (Italic_Vid_Str);
 	if (fgbg & SLTT_BLINK_MASK)
 	  {
 	     /* Someday Linux will have a blink mode that set high intensity
@@ -1493,6 +1678,7 @@ static void write_attributes (SLtt_Char_Type fgbg)
 
    if (SLtt_Use_Ansi_Colors)
      {
+	int bg0, fg0;
 	fg0 = (int) GET_FG(fgbg);
 	bg0 = (int) GET_BG(fgbg);
 
@@ -1501,6 +1687,10 @@ static void write_attributes (SLtt_Char_Type fgbg)
 	  {
 	     if (fg0 == SLSMG_COLOR_DEFAULT)
 	       tt_write_string (Default_Color_Fg_Str);
+#if SLTT_HAS_TRUECOLOR_SUPPORT
+	     else if (IS_TRUE_COLOR(fg0))
+	       write_truecolor (Color_RGB_Fg_Str, fg0);
+#endif
 	     else
 	       tt_printf (Color_Fg_Str, COLOR_ARG(fg0, Is_Fg_BGR), 0);
 	  }
@@ -1510,6 +1700,10 @@ static void write_attributes (SLtt_Char_Type fgbg)
 	  {
 	     if (bg0 == SLSMG_COLOR_DEFAULT)
 	       tt_write_string (Default_Color_Bg_Str);
+#if SLTT_HAS_TRUECOLOR_SUPPORT
+	     else if (IS_TRUE_COLOR(bg0))
+	       write_truecolor (Color_RGB_Bg_Str, bg0);
+#endif
 	     else
 	       tt_printf (Color_Bg_Str, COLOR_ARG(bg0, Is_Bg_BGR), 0);
 	  }
@@ -1533,7 +1727,7 @@ void SLtt_reverse_video (int color)
 	     tt_write_string (Norm_Vid_Str);
 	  }
 	else tt_write_string (Rev_Vid_Str);
-	Current_Fgbg = 0xFFFFFFFFU;
+	Current_Fgbg = INVALID_ATTR;
 	return;
      }
 
@@ -1600,7 +1794,7 @@ static int bce_colors_eq (SLsmg_Color_Type ca, SLsmg_Color_Type cb, int just_bg)
  */
 static void write_string_with_care (SLCONST char *str)
 {
-   unsigned int len;
+   SLstrlen_Type len;
 
    if (str == NULL) return;
 
@@ -1628,7 +1822,7 @@ static void write_string_with_care (SLCONST char *str)
 	       if (SLtt_Screen_Cols > Cursor_c)
 	         {
 		    char *p;
-		    nchars = SLtt_Screen_Cols - Cursor_c - 1;
+		    nchars = (SLstrlen_Type)(SLtt_Screen_Cols - Cursor_c - 1);
 		    p = (char *)SLutf8_skip_chars((SLuchar_Type *) str, (SLuchar_Type *)(str + len), nchars, NULL, 1);
 		    len = p - str;
 		 }
@@ -1823,7 +2017,6 @@ void SLtt_smart_puts(SLsmg_Char_Type *neww, SLsmg_Char_Type *oldd, int len, int
 {
    register SLsmg_Char_Type *p, *q, *qmax, *pmax, *buf;
    SLsmg_Char_Type buffer[SLTT_MAX_SCREEN_COLS+1];
-   unsigned int n_spaces;
    SLsmg_Char_Type *space_match, *last_buffered_match;
 #ifdef HP_GLITCH_CODE
    int handle_hp_glitch = 0;
@@ -2038,7 +2231,7 @@ void SLtt_smart_puts(SLsmg_Char_Type *neww, SLsmg_Char_Type *oldd, int len, int
    while (1)
      {
 	/* while they do not match and we do not hit a space, buffer them up */
-	n_spaces = 0;
+	unsigned int n_spaces = 0;
 	while (p < pmax)
 	  {
 	     if (CHAR_EQS_SPACE(q) && CHAR_EQS_SPACE(p))
@@ -2273,10 +2466,19 @@ void SLtt_smart_puts(SLsmg_Char_Type *neww, SLsmg_Char_Type *oldd, int len, int
 static void get_color_info (void)
 {
    SLCONST char *fg, *bg;
+   char *ct;
 
-   /* Allow easy mechanism to override inadequate termcap/terminfo files. */
-   if (SLtt_Use_Ansi_Colors == 0)
-     SLtt_Use_Ansi_Colors = (NULL != getenv ("COLORTERM"));
+   ct = getenv ("COLORTERM");
+
+   if (ct != NULL)
+     {
+	/* Allow easy mechanism to override inadequate termcap/terminfo files. */
+	SLtt_Use_Ansi_Colors = 1;
+#if SLTT_HAS_TRUECOLOR_SUPPORT
+	if ((0 == strcmp(ct, "truecolor")) || (0 == strcmp(ct, "24bit")))
+	  Has_True_Color = 1;
+#endif
+     }
 
    if (SLtt_Use_Ansi_Colors)
      Is_Color_Terminal = 1;
@@ -2286,6 +2488,8 @@ static void get_color_info (void)
      Can_Background_Color_Erase = (NULL != getenv ("COLORTERM_BCE"));
 #endif
 
+   /* terminfo does not support truecolor */
+
    if (-1 == get_default_colors (&fg, &bg))
      return;
 
@@ -2301,7 +2505,7 @@ static void get_color_info (void)
 
 #ifdef __unix__
 
-static int Termcap_Initalized = 0;
+static int Termcap_Initialized = 0;
 
 /* #define USE_TERMCAP 1 */
 #ifdef USE_TERMCAP
@@ -2363,7 +2567,7 @@ static char *tt_tgetstr (SLCONST char *cap)
    char area_buf[4096];
    char *area;
 #endif
-   if (Termcap_Initalized == 0)
+   if (Termcap_Initialized == 0)
      return NULL;
 
 #ifdef USE_TERMCAP
@@ -2409,7 +2613,7 @@ char *SLtt_tgetstr (SLFUTURE_CONST char *cap)
 
 static int tt_tgetnum (SLCONST char *s)
 {
-   if (Termcap_Initalized == 0)
+   if (Termcap_Initialized == 0)
      return -1;
 #ifdef USE_TERMCAP
    return tgetnum (s);
@@ -2425,7 +2629,7 @@ int SLtt_tgetnum (SLFUTURE_CONST char *s)
 
 static int tt_tgetflag (SLCONST char *s)
 {
-   if (Termcap_Initalized == 0)
+   if (Termcap_Initialized == 0)
      return -1;
 #ifdef USE_TERMCAP
    return tgetflag (s);
@@ -2471,7 +2675,7 @@ void SLtt_get_terminfo (void)
 
    term = getenv ("TERM");
    if (term == NULL)
-     SLang_exit_error("TERM environment variable needs set.");
+     SLang_exit_error("%s", "TERM environment variable needs set.");
 
    if (0 == (status = SLtt_initialize (term)))
      return;
@@ -2547,6 +2751,8 @@ int SLtt_initialize (SLFUTURE_CONST char *term)
    if (Terminfo != NULL)
      _pSLtt_tifreeent (Terminfo);
 
+   Termcap_Initialized = 0;	       /* resetting things */
+
    if (NULL == (Terminfo = _pSLtt_tigetent (term)))
      {
 	if (almost_vtxxx) /* Special cases. */
@@ -2566,7 +2772,7 @@ int SLtt_initialize (SLFUTURE_CONST char *term)
    /* Termcap_String_Ptr = Termcap_String_Buf; */
 # endif				       /* NOT USE_TERMCAP */
 
-   Termcap_Initalized = 1;
+   Termcap_Initialized = 1;
 
    Cls_Str = tt_tgetstr ("cl");
    Abs_Curs_Pos_Str = tt_tgetstr ("cm");
@@ -2665,6 +2871,7 @@ int SLtt_initialize (SLFUTURE_CONST char *term)
      Blink_Vid_Str = "\033[5m";
 
    UnderLine_Vid_Str = tt_tgetstr("us");
+   Italic_Vid_Str = "\033[3m";
 
    Start_Alt_Chars_Str = tt_tgetstr ("as");   /* smacs */
    End_Alt_Chars_Str = tt_tgetstr ("ae");   /* rmacs */
@@ -2862,6 +3069,7 @@ void SLtt_set_term_vtxxx(int *vt100)
    Bold_Vid_Str = "\033[1m";
    Blink_Vid_Str = "\033[5m";
    UnderLine_Vid_Str = "\033[4m";
+   Italic_Vid_Str = "\033[3m";
    Del_Eol_Str = "\033[K";
    Del_Bol_Str = "\033[1K";
    Rev_Scroll_Str = "\033M";
@@ -2968,7 +3176,7 @@ int SLtt_reset_video (void)
    SLtt_normal_video ();	       /* MSKermit requires this  */
    tt_write_string(Norm_Vid_Str);
 
-   Current_Fgbg = 0xFFFFFFFFU;
+   Current_Fgbg = INVALID_ATTR;
    SLtt_set_alt_char_set (0);
    if (SLtt_Use_Ansi_Colors)
      {
@@ -2980,7 +3188,7 @@ int SLtt_reset_video (void)
 	     else tt_write_string ("\033[0m\033[m");
 	  }
 	else tt_write_string (Reset_Color_String);
-	Current_Fgbg = 0xFFFFFFFFU;
+	Current_Fgbg = INVALID_ATTR;
      }
    SLtt_erase_line ();
    SLtt_deinit_keypad ();
@@ -3003,10 +3211,10 @@ void SLtt_bold_video (void)
 
 int SLtt_set_mouse_mode (int mode, int force)
 {
-   char *term;
-
    if (force == 0)
      {
+	char *term;
+
 	if (NULL == (term = (char *) getenv("TERM"))) return -1;
 	if (strncmp ("xterm", term, 5))
 	  return -1;
